import _extends from '@babel/runtime/helpers/extends';
import React, { forwardRef, createContext, useMemo, useState, useContext, useRef, useCallback, useEffect } from 'react';
import classNames from 'classnames';
import { createStyles, mix, readableColor, transparentize, css, readableColorIsBlack, lighten, ThemeProvider as ThemeProvider$1, ColorContextProvider, useTheme, useColorContext, createReadablePalette } from 'react-style-system';
import uid from 'uid';
import { createPortal } from 'react-dom';

const useStyles = createStyles(({
  css,
  theme,
  color,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  return {
    // button base styles
    root: css`
      ${theme.button};

      display: inline-flex;
      justify-content: center;
      align-items: center;

      appearance: none;
      outline: none;
      background: transparent;
      border: none;
      border-radius: ${theme.borderRadius};
      text-decoration: none;
      cursor: pointer;

      &:disabled {
        cursor: not-allowed;
      }
    `,
    // shapes
    standard: css`
      min-width: ${theme.block(1)};

      & > *:not(:last-child) {
        margin-right: ${theme.space(1)};
      }
    `,
    icon: css`
      border-radius: 99999px;
      & > svg {
        width: ${theme.space(1.25)};
        height: ${theme.space(1.25)};
      }
    `,
    // variants
    outlined: css`
      border: 1px solid ${color.readable};
      color: ${color.readable};

      transition: background-color ${theme.duration.standard},
        border ${theme.duration.standard};

      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.92)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
      }
      &:active {
        background-color: ${mixWithSurface(color.decorative, 0.8)};
      }
      &:disabled {
        color: ${mixWithSurface(color.readable, 0.4)};
        background-color: transparent;
      }
    `,
    filled: css`
      background-color: ${color.decorative};
      color: ${readableColor(color.decorative)} !important;
      border: 2px solid transparent;
      transition: background-color ${theme.duration.standard},
        border ${theme.duration.standard};

      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.18)};
        border: 2px solid ${transparentize('#fff', 0.7)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.23)};
      }
      &:active {
        background-color: ${mixWithSurface(color.decorative, 0.3)};
        border: 2px solid ${transparentize('#fff', 0.5)};
      }
      &:disabled {
        background-color: ${mixWithSurface(color.decorative, 0.5)};
        border: 2px solid transparent;
        color: ${mixWithSurface(readableColor(color.decorative), 0.2)};
      }
    `,
    ghost: css`
      color: ${color.readable};
      border: 2px solid transparent;
      transition: background-color ${theme.duration.standard},
        border ${theme.duration.standard};

      &:focus {
        background-color: ${transparentize(color.decorative, 0.92)};
        /* TODO: there's a bug in react-style-system that requires prettier-ignore */
        /* prettier-ignore */
        border: 2px solid ${transparentize(readableColor(color.decorative), 0.8)};
      }
      &:hover {
        background-color: ${transparentize(color.decorative, 0.9)};
      }
      &:active {
        background-color: ${transparentize(color.decorative, 0.8)};
      }
      &:disabled {
        color: ${transparentize(color.readable, 0.4)};
        background-color: transparent;
      }
    `,
    // sizes
    standardSmall: css`
      padding: ${theme.space(0.5)} ${theme.space(0.75)};
      min-width: ${theme.block(0.75)};
    `,
    standardStandard: css`
      padding: ${theme.space(0.75)} ${theme.space(1)};
      min-width: ${theme.block(1)};
    `,
    standardLarge: css`
      padding: ${theme.space(1)} ${theme.space(1.25)};
      min-width: ${theme.block(1.5)};
      font-size: 0.8rem;
    `,
    iconSmall: css`
      width: ${theme.space(2)};
      height: ${theme.space(2)};
    `,
    iconStandard: css`
      width: ${theme.space(2.5)};
      height: ${theme.space(2.5)};
    `,
    iconLarge: css`
      width: ${theme.space(3)};
      height: ${theme.space(3)};
    `
  };
});
const Button = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    component,
    variant = 'ghost',
    size = 'standard',
    shape = 'standard',
    ...restOfProps
  } = useStyles(props, props.component || 'button');
  return /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames('hui-button', {
      [styles.icon]: shape === 'icon',
      [styles.standard]: shape === 'standard',
      [styles.outlined]: variant === 'outlined',
      [styles.ghost]: variant === 'ghost',
      [styles.filled]: variant === 'filled',
      [styles.iconSmall]: shape === 'icon' && size === 'small',
      [styles.iconStandard]: shape === 'icon' && size === 'standard',
      [styles.iconLarge]: shape === 'icon' && size === 'large',
      [styles.standardSmall]: shape === 'standard' && size === 'small',
      [styles.standardStandard]: shape === 'standard' && size === 'standard',
      [styles.standardLarge]: shape === 'standard' && size === 'large'
    }),
    ref: ref
  }, restOfProps));
});
Button.displayName = 'Button';

const useStyles$1 = createStyles(({
  css
}) => ({
  root: css`
    list-style-type: none;
    padding: 0;
    margin: 0;
  `
}));
const List = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    ...restOfProps
  } = useStyles$1(props, 'ul' );
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
List.displayName = 'List';

const useStyles$2 = createStyles(({
  css,
  theme
}) => ({
  root: css`
    ${theme.body2};
    display: flex;
  `
}));
const ListItem = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    ...restOfProps
  } = useStyles$2(props, 'li');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
ListItem.displayName = 'ListItem';

const parsePx = value => {
  const match = value.match(/([\d.]+)px.*/i);

  if (!match) {
    throw new Error(`Expected value to have px units. Got: "${value}"`);
  }

  return parseFloat(match[1]);
};

const defaultTheme = {
  // media queries
  media: {
    up(value) {
      console.log('this value', this[value]);
      return `@media (min-width: ${parsePx(this[value]) + 1}px)`;
    },

    down(value) {
      console.log('this value', this[value]);
      return `@media (max-width: ${this[value]})`;
    },

    between(min, max) {
      return `@media (max-width: ${this[min]}) and (min-width: ${parsePx(this[max]) + 1}px)`;
    },

    mobile: '425px',
    tablet: '768px',
    desktop: '1024px',
    desktopLarge: '1440px'
  },
  // non-responsive font-sizes
  fontStatic: {
    h1: css`
      font-size: 144px;
      letter-spacing: -3px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    h2: css`
      font-size: 96px;
      letter-spacing: -2px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    h3: css`
      font-size: 60px;
      letter-spacing: -0.3px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    h4: css`
      font-size: 42px;
      letter-spacing: -0.3px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    h5: css`
      font-size: 28px;
      letter-spacing: -0.3px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    h6: css`
      font-size: 18px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    subtitle1: css`
      font-size: 16px;
      text-transform: none;
      margin: 0;
    `,
    subtitle2: css`
      font-size: 14px;
      font-weight: bold;
      text-transform: none;
      margin: 0;
    `,
    body1: css`
      font-size: 18px;
      line-height: 1.6;
      letter-spacing: -0.333px;
      text-transform: none;
      font-weight: 400;
      margin: 0;
    `,
    body2: css`
      font-size: 16px;
      text-transform: none;
      line-height: 1.5;
      font-weight: 400;
      margin: 0;
    `,
    button: css`
      font-size: 0.75em;
      font-weight: bold;
      text-transform: uppercase;
      margin: 0;
    `,
    caption: css`
      font-size: 14px;
      font-weight: normal;
      text-transform: none;
      margin: 0;
    `,
    overline: css`
      font-size: 12px;
      font-weight: 400;
      letter-spacing: 1.2px;
      text-transform: uppercase;
      margin: 0;
    `
  },
  weight: {
    light: 300,
    normal: 400,
    semiBold: 600,
    bold: 700,
    black: 900
  },

  // responsive/default font-sizes
  get h1() {
    return css`
      ${this.fontStatic.h1};
      ${this.media.down('tablet')} {
        ${this.fontStatic.h2};
      }
    `;
  },

  get h2() {
    return css`
      ${this.fontStatic.h2};
      ${this.media.down('tablet')} {
        ${this.fontStatic.h3};
      }
    `;
  },

  get h3() {
    return css`
      ${this.fontStatic.h3};
      ${this.media.down('tablet')} {
        ${this.fontStatic.h4};
      }
    `;
  },

  get h4() {
    return css`
      ${this.fontStatic.h4};
      ${this.media.down('tablet')} {
        ${this.fontStatic.h5};
      }
    `;
  },

  get h5() {
    return this.fontStatic.h5;
  },

  get h6() {
    return this.fontStatic.h6;
  },

  get subtitle1() {
    return this.fontStatic.subtitle1;
  },

  get subtitle2() {
    return this.fontStatic.subtitle2;
  },

  get body1() {
    return css`
      ${this.fontStatic.body1};
      ${this.media.down('mobile')} {
        ${this.fontStatic.body2};
      }
    `;
  },

  get body2() {
    return this.fontStatic.body2;
  },

  get caption() {
    return this.fontStatic.caption;
  },

  get button() {
    return this.fontStatic.button;
  },

  get overline() {
    return this.fontStatic.overline;
  },

  // colors
  surface: '#fff',
  brand: '#000',

  get accent() {
    const accent = '#2962ff';
    return readableColorIsBlack(this.surface) ? accent : lighten(accent, 0.1);
  },

  get danger() {
    const danger = '#eb002b';
    return readableColorIsBlack(this.surface) ? danger : lighten(danger, 0.2);
  },

  warning: '#f56200',

  get bland() {
    return readableColorIsBlack(this.surface) ? '#ccc' : '#555';
  },

  // z-indexes
  zIndex: {
    appBar: 1100,
    drawer: 1200,
    modal: 1300,
    notification: 1400,
    tooltip: 1500
  },
  // border-radius
  borderRadius: '2px',
  // spacing unit helpers
  space: n => `${n * 16}px`,
  gap: n => `${n * 48}px`,
  block: n => `${n * 96}px`,
  // durations
  duration: {
    short: '100ms',
    standard: '250ms',
    long: '500ms'
  },
  // shadows
  shadows: {
    subtle: '0 0 10px 0 rgba(0,0,0,0.10)',
    standard: '0 0 20px 0 rgba(0,0,0,0.13)',
    emphasis: '0 0 30px 0 rgba(0,0,0,0.13)'
  }
};

function ThemeProvider({
  theme = defaultTheme,
  children
}) {
  return /*#__PURE__*/React.createElement(ThemeProvider$1, {
    theme: theme
  }, /*#__PURE__*/React.createElement(ColorContextProvider, {
    color: theme.accent,
    surface: theme.surface
  }, children));
}

function isObject(maybe) {
  if (!maybe) return false;
  return typeof maybe === 'object';
}
/**
 * recursively merges a base object with an incoming object
 */


function recursiveMerge(base, incoming) {
  const keys = [...Object.keys(base), ...Object.keys(incoming)];
  const merged = keys.reduce((merged, key) => {
    var _baseDescriptor$get, _incomingDescriptor$g;

    const baseValue = base[key];
    const incomingValue = incoming[key];

    if (isObject(baseValue) || isObject(incomingValue)) {
      merged[key] = recursiveMerge(baseValue || {}, incomingValue || {});
      return merged;
    }

    const baseDescriptor = Object.getOwnPropertyDescriptor(base, key);
    const incomingDescriptor = Object.getOwnPropertyDescriptor(incoming, key);
    const baseGetter = baseDescriptor === null || baseDescriptor === void 0 ? void 0 : (_baseDescriptor$get = baseDescriptor.get) === null || _baseDescriptor$get === void 0 ? void 0 : _baseDescriptor$get.bind(merged);
    const incomingGetter = incomingDescriptor === null || incomingDescriptor === void 0 ? void 0 : (_incomingDescriptor$g = incomingDescriptor.get) === null || _incomingDescriptor$g === void 0 ? void 0 : _incomingDescriptor$g.bind(merged);
    const incomingIsValue = incomingDescriptor && 'value' in incomingDescriptor || false;

    if (incomingIsValue) {
      // TODO: test if this is needed
      if (typeof incomingValue === 'function') {
        merged[key] = incomingValue.bind(merged);
        return merged;
      }

      merged[key] = incomingValue;
      return merged;
    }

    if (incomingGetter) {
      Object.defineProperty(merged, key, {
        get: incomingGetter,
        enumerable: true,
        configurable: true
      });
      return merged;
    }

    if (baseGetter) {
      Object.defineProperty(merged, key, {
        get: baseGetter,
        enumerable: true,
        configurable: true
      });
      return merged;
    } // TODO: test if this is needed


    if (typeof baseValue === 'function') {
      merged[key] = baseValue.bind(merged);
      return merged;
    }

    merged[key] = baseValue;
    return merged;
  }, {});
  return merged;
}

function createTheme(partialTheme) {
  return recursiveMerge(defaultTheme, partialTheme || {});
}

const useStyles$3 = createStyles(({
  css,
  theme
}) => ({
  root: css`
    ${theme.body2};
    align-items: flex-start;
    width: 100%;
    display: flex;
    flex-direction: column;
  `
}));
const ListItemButton = forwardRef((props, ref) => {
  const theme = useTheme();
  const {
    Root,
    styles,
    ...restOfProps
  } = useStyles$3(props, Button);
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref,
    variant: "ghost",
    color: theme.bland
  }, restOfProps));
});
ListItemButton.displayName = 'ListItemButton';

const useStyles$4 = createStyles(({
  css,
  theme,
  color,
  surface
}) => ({
  root: css`
    color: ${color.readable};
    text-decoration: none;
    transition: color ${theme.duration.standard};

    &:hover {
      text-decoration: underline;
    }

    &:active {
      color: ${mix(color.readable, surface, 0.3)};
    }
  `
}));
const Anchor = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    ...restOfProps
  } = useStyles$4(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'a');
  return /*#__PURE__*/React.createElement(Root, _extends({
    className: "hui-anchor",
    ref: ref
  }, restOfProps));
});
Anchor.displayName = 'Anchor';

var FormControlContext = createContext(null);

const useStyles$5 = createStyles(({
  css
}) => ({
  root: css`
    display: flex;
    flex-direction: column;
  `
}));
const FormControl = forwardRef((props, ref) => {
  var _ref, _props$color, _ref2, _props$surface;

  const {
    Root,
    styles,
    component,
    hasError = false,
    disabled = false,
    ...restOfProps
  } = useStyles$5(props, props.component || 'div');
  const colorContext = useColorContext();
  const theme = useTheme();
  const color = (_ref = (_props$color = props.color) !== null && _props$color !== void 0 ? _props$color : colorContext === null || colorContext === void 0 ? void 0 : colorContext.color.original) !== null && _ref !== void 0 ? _ref : theme.accent;
  const surface = (_ref2 = (_props$surface = props.surface) !== null && _props$surface !== void 0 ? _props$surface : colorContext === null || colorContext === void 0 ? void 0 : colorContext.surface) !== null && _ref2 !== void 0 ? _ref2 : theme.surface;
  const id = useMemo(() => `hui-${uid()}`, []);
  const [focused, setFocused] = useState(false);
  const formControlContextValue = useMemo(() => ({
    id,
    focused,
    setFocused,
    hasError,
    disabled
  }), [disabled, focused, hasError, id]);
  return /*#__PURE__*/React.createElement(ColorContextProvider, {
    color: color,
    surface: surface
  }, /*#__PURE__*/React.createElement(FormControlContext.Provider, {
    value: formControlContextValue
  }, /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps))));
});
FormControl.displayName = 'FormControl';

const useStyles$6 = createStyles(({
  css,
  theme,
  color,
  surface
}) => {
  const bland = createReadablePalette(theme.bland, surface);
  return {
    root: css`
      ${theme.body1};
      color: ${readableColor(theme.surface)};
      transition: color ${theme.duration.standard};
      cursor: pointer;
    `,
    focused: css`
      color: ${color.readable};
    `,
    hasError: css`
      color: ${theme.danger};
    `,
    disabled: css`
      color: ${mix(bland.readable, surface, 0.3)};
      cursor: not-allowed;
    `
  };
});
const Label = forwardRef((props, ref) => {
  var _ref, _ref2, _ref3;

  const {
    Root,
    styles,
    component,
    id: incomingId,
    focused: incomingFocused,
    hasError: incomingHasError,
    disabled: incomingDisabled,
    ...restOfProps
  } = useStyles$6(props, props.component || 'label');
  const formControlContext = useContext(FormControlContext);
  const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
  const focused = (_ref = incomingFocused !== null && incomingFocused !== void 0 ? incomingFocused : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.focused) !== null && _ref !== void 0 ? _ref : false;
  const hasError = (_ref2 = incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) !== null && _ref2 !== void 0 ? _ref2 : false;
  const disabled = (_ref3 = incomingDisabled !== null && incomingDisabled !== void 0 ? incomingDisabled : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) !== null && _ref3 !== void 0 ? _ref3 : false;
  return /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.focused]: focused,
      [styles.hasError]: hasError,
      [styles.disabled]: disabled
    }),
    ref: ref,
    htmlFor: id
  }, restOfProps));
});
Label.displayName = 'Label';

const useStyles$7 = createStyles(({
  css,
  theme,
  color,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  const bland = createReadablePalette(theme.bland, surface);
  const danger = createReadablePalette(theme.danger, surface);
  return {
    root: css`
      ${theme.body1};
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border: none;
      outline: none;
      appearance: none;
      background: none;
      margin: ${theme.space(0.5)} 0;
      border-radius: ${theme.borderRadius};
      color: ${readableColor(theme.surface)};

      &::placeholder {
        color: ${mixWithSurface(readableColor(theme.surface), 0.25)};
      }

      &:disabled {
        cursor: not-allowed;
      }
    `,
    filled: css`
      background-color: ${mixWithSurface(bland.decorative, 0.8)};
      color: ${readableColor(surface)};
      transition: background-color ${theme.duration.standard};

      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.92)};
        color: ${readableColor(surface)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
        color: ${readableColor(surface)};
      }
      &:disabled {
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }
    `,
    filledHasError: css`
      background-color: ${mixWithSurface(danger.decorative, 0.9)};
      color: ${readableColor(surface)};
      &:focus {
        background-color: ${mixWithSurface(danger.decorative, 0.85)};
      }
      &:not([disabled]):hover {
        background-color: ${mixWithSurface(danger.decorative, 0.87)};
      }
    `,
    outlined: css`
      background-color: ${surface};
      transition: border ${theme.duration.standard},
        background-color ${theme.duration.standard};
      border: 2px solid ${bland.decorative};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:hover {
        border: 2px solid ${transparentize(color.decorative, 0.3)};
      }
      &:disabled {
        border: 2px solid ${transparentize(bland.decorative, 0.7)};
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }
    `,
    outlinedHasError: css`
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${transparentize(danger.decorative, 0.3)};
      }
    `
  };
});
const TextInput = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    variant = 'outlined',
    hasError: incomingHasError,
    disabled: incomingDisabled,
    onFocus,
    onBlur,
    type = 'text',
    ...restOfProps
  } = useStyles$7(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'input');
  const formControlContext = useContext(FormControlContext);
  const id = formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
  const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
  const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);

  const handleFocus = e => {
    if (onFocus) {
      onFocus(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(true);
    }
  };

  const handleBlur = e => {
    if (onBlur) {
      onBlur(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(false);
    }
  };

  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref,
    id: id,
    type: type,
    disabled: disabled,
    className: classNames({
      [styles.outlined]: variant === 'outlined',
      [styles.outlinedHasError]: variant === 'outlined' && hasError,
      [styles.filled]: variant === 'filled',
      [styles.filledHasError]: variant === 'filled' && hasError
    }),
    onFocus: handleFocus,
    onBlur: handleBlur
  }, restOfProps));
});
TextInput.displayName = 'TextInput';

const useStyles$8 = createStyles(({
  css,
  color,
  theme,
  surface
}) => {
  const bland = createReadablePalette(theme.bland, surface);
  return {
    root: css`
      ${theme.caption};
      transition: color ${theme.duration.standard};
    `,
    focused: css`
      color: ${color.readable};
    `,
    hasError: css`
      color: ${theme.danger};
    `,
    disabled: css`
      color: ${mix(bland.readable, surface, 0.3)};
      cursor: not-allowed;
    `
  };
});
const HelperText = forwardRef((props, ref) => {
  var _props$component, _ref, _ref2, _ref3;

  const formControlContext = useContext(FormControlContext);
  const {
    Root,
    styles,
    component,
    focused: incomingFocused,
    hasError: incomingHasError,
    disabled: incomingDisabled,
    ...restOfProps
  } = useStyles$8(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  const focused = (_ref = incomingFocused !== null && incomingFocused !== void 0 ? incomingFocused : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.focused) !== null && _ref !== void 0 ? _ref : false;
  const hasError = (_ref2 = incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) !== null && _ref2 !== void 0 ? _ref2 : false;
  const disabled = (_ref3 = incomingDisabled !== null && incomingDisabled !== void 0 ? incomingDisabled : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) !== null && _ref3 !== void 0 ? _ref3 : false;
  return /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.focused]: focused,
      [styles.hasError]: hasError,
      [styles.disabled]: disabled
    }),
    ref: ref
  }, restOfProps));
});
HelperText.displayName = 'HelperText';

const useStyles$9 = createStyles(({
  css,
  theme,
  color,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  const bland = createReadablePalette(theme.bland, surface);
  const danger = createReadablePalette(theme.danger, surface);
  return {
    root: css`
      ${theme.body1};
      color: ${readableColor(theme.surface)};
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border: none;
      outline: none;
      appearance: none;
      background: none;
      margin: ${theme.space(0.5)} 0;
      border-radius: ${theme.borderRadius};

      &::placeholder {
        color: ${mixWithSurface(readableColor(theme.surface), 0.25)};
      }

      &:disabled {
        cursor: not-allowed;
      }
    `,
    filled: css`
      background-color: ${mixWithSurface(bland.decorative, 0.8)};
      color: ${readableColor(surface)};
      transition: background-color ${theme.duration.standard};

      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.92)};
        color: ${readableColor(surface)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
        color: ${readableColor(surface)};
      }
      &:disabled {
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }
    `,
    filledHasError: css`
      background-color: ${mixWithSurface(danger.decorative, 0.9)};
      color: ${readableColor(surface)};
      &:focus {
        background-color: ${mixWithSurface(danger.decorative, 0.85)};
      }
      &:not([disabled]):hover {
        background-color: ${mixWithSurface(danger.decorative, 0.87)};
      }
    `,
    outlined: css`
      background-color: ${surface};
      transition: border ${theme.duration.standard},
        background-color ${theme.duration.standard};
      border: 2px solid ${bland.decorative};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:hover {
        border: 2px solid ${transparentize(color.decorative, 0.3)};
      }
      &:disabled {
        border: 2px solid ${transparentize(bland.decorative, 0.7)};
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }
    `,
    outlinedHasError: css`
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${transparentize(danger.decorative, 0.3)};
      }
    `
  };
});
const TextArea = forwardRef((props, ref) => {
  var _props$component, _ref, _formControlContext$d;

  const {
    Root,
    styles,
    variant = 'outlined',
    focused: _focused,
    hasError: incomingHasError,
    disabled: incomingDisabled,
    onFocus,
    onBlur,
    ...restOfProps
  } = useStyles$9(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'textarea');
  const formControlContext = useContext(FormControlContext);
  const id = formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
  const hasError = (_ref = incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) !== null && _ref !== void 0 ? _ref : false;
  const disabledFromFormControl = (_formControlContext$d = formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) !== null && _formControlContext$d !== void 0 ? _formControlContext$d : false;
  const disabled = incomingDisabled || disabledFromFormControl;

  const handleFocus = e => {
    if (onFocus) {
      onFocus(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(true);
    }
  };

  const handleBlur = e => {
    if (onBlur) {
      onBlur(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(false);
    }
  };

  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref,
    id: id,
    disabled: disabled,
    className: classNames({
      [styles.outlined]: variant === 'outlined',
      [styles.outlinedHasError]: variant === 'outlined' && hasError,
      [styles.filled]: variant === 'filled',
      [styles.filledHasError]: variant === 'filled' && hasError
    }),
    onFocus: handleFocus,
    onBlur: handleBlur
  }, restOfProps));
});
TextArea.displayName = 'TextArea';

const CheckIcon = forwardRef((props, ref) => {
  return /*#__PURE__*/React.createElement("svg", _extends({
    ref: ref,
    "aria-hidden": "true",
    viewBox: "0 0 512 512"
  }, props), /*#__PURE__*/React.createElement("path", {
    fill: "currentColor",
    d: "M173.898 439.404l-166.4-166.4c-9.997-9.997-9.997-26.206 0-36.204l36.203-36.204c9.997-9.998 26.207-9.998 36.204 0L192 312.69 432.095 72.596c9.997-9.997 26.207-9.997 36.204 0l36.203 36.204c9.997 9.997 9.997 26.206 0 36.204l-294.4 294.401c-9.998 9.997-26.207 9.997-36.204-.001z"
  }));
});
CheckIcon.displayName = 'CheckIcon';

const useStyles$a = createStyles(({
  css,
  color,
  theme,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  const bland = createReadablePalette(theme.bland, surface);
  const danger = createReadablePalette(theme.danger, surface);
  const width = theme.space(2);
  const height = theme.space(2);
  const widthSmall = theme.space(1.5);
  const heightSmall = theme.space(1.5);
  const widthLarge = theme.space(2.5);
  const heightLarge = theme.space(2.5);
  return {
    root: css`
      position: relative;
      margin: ${theme.space(0.5)} 0;
      background-color: ${surface};
    `,
    hasError: css`
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .checkbox:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      & .checkbox:not([disabled]):hover ~ .facade {
        border: 2px solid ${transparentize(danger.decorative, 0.3)};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      & .checkbox:active ~ .facade {
        background-color: ${mixWithSurface(danger.decorative, 0.9)};
      }
    `,
    checkbox: css`
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:hover ~ .facade {
        border: 2px solid ${transparentize(color.decorative, 0.3)};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:active ~ .facade {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${transparentize(bland.decorative, 0.7)};
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .icon {
        opacity: 1;
      }
    `,
    checkboxStandard: css`
      width: ${width};
      height: ${height};
    `,
    checkboxSmall: css`
      width: ${widthSmall};
      height: ${heightSmall};
    `,
    checkboxLarge: css`
      width: ${widthLarge};
      height: ${heightLarge};
    `,
    facade: css`
      width: ${width};
      height: ${height};
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.duration.standard},
        background-color ${theme.duration.standard};
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
      border-radius: ${theme.borderRadius};
    `,
    facadeSmall: css`
      width: ${widthSmall};
      height: ${heightSmall};
    `,
    facadeStandard: css`
      width: ${width};
      height: ${height};
    `,
    facadeLarge: css`
      width: ${widthLarge};
      height: ${heightLarge};
    `,
    icon: css`
      width: 1rem;
      height: 1rem;
      fill: currentColor;
      opacity: 0;
      pointer-events: none;
    `
  };
});
const Checkbox = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    id: incomingId,
    hasError: incomingHasError,
    disabled: incomingDisabled,
    onFocus,
    onBlur,
    inputRef,
    icon: Icon = CheckIcon,
    size = 'standard',
    ...restOfProps
  } = useStyles$a(props);
  const formControlContext = useContext(FormControlContext);
  const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
  const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
  const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);

  const handleFocus = e => {
    if (onFocus) {
      onFocus(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(true);
    }
  };

  const handleBlur = e => {
    if (onBlur) {
      onBlur(e);
    }

    if (formControlContext) {
      formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.setFocused(false);
    }
  };

  return /*#__PURE__*/React.createElement(Root, {
    ref: ref,
    className: classNames({
      [styles.hasError]: hasError
    })
  }, /*#__PURE__*/React.createElement("input", _extends({
    type: "checkbox",
    id: id,
    ref: inputRef,
    className: classNames('checkbox', styles.checkbox, {
      [styles.checkboxSmall]: size === 'small',
      [styles.checkboxStandard]: size === 'standard',
      [styles.checkboxLarge]: size === 'large'
    }),
    onFocus: handleFocus,
    onBlur: handleBlur,
    disabled: disabled
  }, restOfProps)), /*#__PURE__*/React.createElement("div", {
    "aria-hidden": "true",
    className: classNames('facade', styles.facade, {
      [styles.facadeSmall]: size === 'small',
      [styles.facadeStandard]: size === 'standard',
      [styles.facadeLarge]: size === 'large'
    })
  }, /*#__PURE__*/React.createElement(Icon, {
    className: classNames('icon', styles.icon)
  })));
});
Checkbox.displayName = 'Checkbox';

var RadioGroupContext = createContext(null);

const CircleIcon = forwardRef((props, ref) => {
  return /*#__PURE__*/React.createElement("svg", _extends({
    ref: ref,
    "aria-hidden": "true",
    viewBox: "0 0 512 512"
  }, props), /*#__PURE__*/React.createElement("circle", {
    cx: "256",
    cy: "256",
    fill: "currentColor",
    r: "192"
  }));
});
CircleIcon.displayName = 'CircleIcon';

const useStyles$b = createStyles(({
  css,
  color,
  theme,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  const bland = createReadablePalette(theme.bland, surface);
  const danger = createReadablePalette(theme.danger, surface);
  const width = theme.space(2);
  const height = theme.space(2);
  const widthSmall = theme.space(1.5);
  const heightSmall = theme.space(1.5);
  const widthLarge = theme.space(2.5);
  const heightLarge = theme.space(2.5);
  return {
    root: css`
      position: relative;
      margin: ${theme.space(0.5)} 0;
    `,
    hasError: css`
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .radio:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      & .radio:not([disabled]):hover ~ .facade {
        border: 2px solid ${transparentize(danger.decorative, 0.3)};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      & .radio:active ~ .facade {
        background-color: ${mixWithSurface(danger.decorative, 0.9)};
      }
    `,
    radio: css`
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:hover ~ .facade {
        border: 2px solid ${transparentize(color.decorative, 0.3)};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:active ~ .facade {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${transparentize(bland.decorative, 0.7)};
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .icon {
        opacity: 1;
      }
    `,
    radioStandard: css`
      width: ${width};
      height: ${height};
    `,
    radioSmall: css`
      width: ${widthSmall};
      height: ${heightSmall};
    `,
    radioLarge: css`
      width: ${widthLarge};
      height: ${heightLarge};
    `,
    facade: css`
      width: ${width};
      height: ${height};
      border-radius: 99999px;
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.duration.standard},
        background-color ${theme.duration.standard};
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
    facadeSmall: css`
      width: ${widthSmall};
      height: ${heightSmall};
    `,
    facadeStandard: css`
      width: ${width};
      height: ${height};
    `,
    facadeLarge: css`
      width: ${widthLarge};
      height: ${heightLarge};
    `,
    icon: css`
      width: 1rem;
      height: 1rem;
      fill: currentColor;
      opacity: 0;
      pointer-events: none;
    `
  };
});
const Radio = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    id: incomingId,
    hasError: incomingHasError,
    disabled: incomingDisabled,
    checked: incomingChecked,
    name: incomingName,
    value,
    onFocus,
    onBlur,
    onChange,
    inputRef,
    icon: Icon = CircleIcon,
    size = 'standard',
    ...restOfProps
  } = useStyles$b(props);
  const formControlContext = useContext(FormControlContext);
  const radioGroupContext = useContext(RadioGroupContext);
  const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
  const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
  const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
  const checkedFromContext = radioGroupContext ? radioGroupContext.value === value : undefined;
  const checked = incomingChecked !== null && incomingChecked !== void 0 ? incomingChecked : checkedFromContext;
  const name = incomingName !== null && incomingName !== void 0 ? incomingName : radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.name;

  const handleFocus = e => {
    if (onFocus) {
      onFocus(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(true);
    }
  };

  const handleBlur = e => {
    if (onBlur) {
      onBlur(e);
    }

    if (formControlContext) {
      formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.setFocused(false);
    }
  };

  const handleChange = e => {
    if (onChange) {
      onChange(e);
    }

    if (radioGroupContext) {
      radioGroupContext.handleChange(e);
    }
  };

  return /*#__PURE__*/React.createElement(Root, {
    ref: ref,
    className: classNames({
      [styles.hasError]: hasError
    })
  }, /*#__PURE__*/React.createElement("input", _extends({
    type: "radio",
    id: id,
    ref: inputRef,
    value: value,
    name: name,
    checked: checked,
    className: classNames('radio', styles.radio, {
      [styles.radioSmall]: size === 'small',
      [styles.radioStandard]: size === 'standard',
      [styles.radioLarge]: size === 'large'
    }),
    onFocus: handleFocus,
    onBlur: handleBlur,
    onChange: handleChange,
    disabled: disabled
  }, restOfProps)), /*#__PURE__*/React.createElement("div", {
    "aria-hidden": "true",
    className: classNames('facade', styles.facade, {
      [styles.facadeSmall]: size === 'small',
      [styles.facadeStandard]: size === 'standard',
      [styles.facadeLarge]: size === 'large'
    })
  }, /*#__PURE__*/React.createElement(Icon, {
    className: classNames('icon', styles.icon)
  })));
});
Radio.displayName = 'Radio';

const useStyles$c = createStyles(({
  css
}) => ({
  root: css``
}));
const RadioGroup = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    value: incomingValue,
    onChange,
    ...restOfProps
  } = useStyles$c(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  const name = useMemo(() => `hui-group-${uid()}`, []);
  const [stateValue, setStateValue] = useState('');
  const value = incomingValue !== null && incomingValue !== void 0 ? incomingValue : stateValue; // pull the reference of onChange on invocation

  const onChangeRef = useRef(onChange);
  const handleChange = useCallback(e => {
    const onChange = onChangeRef.current;

    if (onChange) {
      onChange(e);
    }

    setStateValue(e.currentTarget.value);
  }, []);
  const contextValue = useMemo(() => ({
    name,
    value,
    handleChange
  }), [handleChange, name, value]);
  return /*#__PURE__*/React.createElement(RadioGroupContext.Provider, {
    value: contextValue
  }, /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref,
    role: "radiogroup"
  }, restOfProps)));
});
RadioGroup.displayName = 'RadioGroup';

const useStyles$d = createStyles(({
  css,
  color,
  theme,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  const bland = createReadablePalette(theme.bland, surface);
  const danger = createReadablePalette(theme.danger, surface);
  const width = theme.space(3.5);
  const height = theme.space(2);
  const widthSmall = theme.space(2.75);
  const heightSmall = theme.space(1.5);
  const widthLarge = theme.space(4);
  const heightLarge = theme.space(2.5);
  return {
    root: css`
      position: relative;
      margin: ${theme.space(0.5)} 0;
      background-color: ${surface};
      border-radius: 99999px;
    `,
    hasError: css`
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .switch:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      & .switch:not([disabled]):hover ~ .facade {
        border: 2px solid ${transparentize(danger.decorative, 0.3)};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      & .switch:active ~ .facade {
        background-color: ${mixWithSurface(danger.decorative, 0.9)};
      }

      & .switch:checked ~ .facade {
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }

      & .switch:checked ~ .facade .dot {
        fill: ${danger.decorative};
      }
    `,
    switch: css`
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:hover ~ .facade {
        border: 2px solid ${transparentize(color.decorative, 0.3)};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:active ~ .facade {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${transparentize(bland.decorative, 0.7)};
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .dot {
        cx: 125;
        fill: ${color.decorative};
      }

      &:disabled ~ .facade .dot {
        fill: ${bland.decorative};
      }

      &:checked ~ .facade {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
      }
    `,
    switchStandard: css`
      width: ${width};
      height: ${height};
    `,
    switchSmall: css`
      width: ${widthSmall};
      height: ${heightSmall};
    `,
    switchLarge: css`
      width: ${widthLarge};
      height: ${heightLarge};
    `,
    facade: css`
      width: ${width};
      height: ${height};
      border-radius: 99999px;
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.duration.standard},
        background-color ${theme.duration.standard};
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
    facadeSmall: css`
      width: ${widthSmall};
      height: ${heightSmall};
    `,
    facadeStandard: css`
      width: ${width};
      height: ${height};
    `,
    facadeLarge: css`
      width: ${widthLarge};
      height: ${heightLarge};
    `,
    icon: css`
      width: ${width};
      height: ${height};
      fill: currentColor;
      pointer-events: none;
    `,
    dot: css`
      transition: cx ${theme.duration.standard}, fill ${theme.duration.standard};
    `,
    dotSmall: css`
      r: 30;
    `,
    dotStandard: css`
      r: 35;
    `,
    dotLarge: css`
      r: 40;
    `
  };
});
const Switch = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    id: incomingId,
    hasError: incomingHasError,
    disabled: incomingDisabled,
    onFocus,
    onBlur,
    inputRef,
    size = 'standard',
    ...restOfProps
  } = useStyles$d(props);
  const formControlContext = useContext(FormControlContext);
  const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
  const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
  const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);

  const handleFocus = e => {
    if (onFocus) {
      onFocus(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(true);
    }
  };

  const handleBlur = e => {
    if (onBlur) {
      onBlur(e);
    }

    if (formControlContext) {
      formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.setFocused(false);
    }
  };

  return /*#__PURE__*/React.createElement(Root, {
    ref: ref,
    className: classNames({
      [styles.hasError]: hasError
    })
  }, /*#__PURE__*/React.createElement("input", _extends({
    type: "checkbox",
    id: id,
    ref: inputRef,
    className: classNames('switch', styles.switch, {
      [styles.switchSmall]: size === 'small',
      [styles.switchStandard]: size === 'standard',
      [styles.switchLarge]: size === 'large'
    }),
    onFocus: handleFocus,
    onBlur: handleBlur,
    disabled: disabled
  }, restOfProps)), /*#__PURE__*/React.createElement("div", {
    "aria-hidden": "true",
    className: classNames('facade', styles.facade, {
      [styles.facadeSmall]: size === 'small',
      [styles.facadeStandard]: size === 'standard',
      [styles.facadeLarge]: size === 'large'
    })
  }, /*#__PURE__*/React.createElement("svg", {
    className: styles.icon,
    viewBox: "0 0 175 100"
  }, /*#__PURE__*/React.createElement("circle", {
    className: classNames('dot', styles.dot, {
      [styles.dotSmall]: size === 'small',
      [styles.dotStandard]: size === 'standard',
      [styles.dotLarge]: size === 'large'
    }),
    cx: "50",
    cy: "50"
  }))));
});
Switch.displayName = 'Switch';

const AngleDownIcon = forwardRef((props, ref) => {
  return /*#__PURE__*/React.createElement("svg", _extends({
    ref: ref,
    "aria-hidden": "true",
    viewBox: "0 0 320 512"
  }, props), /*#__PURE__*/React.createElement("path", {
    fill: "currentColor",
    d: "M143 352.3L7 216.3c-9.4-9.4-9.4-24.6 0-33.9l22.6-22.6c9.4-9.4 24.6-9.4 33.9 0l96.4 96.4 96.4-96.4c9.4-9.4 24.6-9.4 33.9 0l22.6 22.6c9.4 9.4 9.4 24.6 0 33.9l-136 136c-9.2 9.4-24.4 9.4-33.8 0z"
  }));
});
AngleDownIcon.displayName = 'AngleDownIcon';

const useStyles$e = createStyles(({
  css,
  theme,
  color,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  const bland = createReadablePalette(theme.bland, surface);
  const danger = createReadablePalette(theme.danger, surface);
  return {
    root: css`
      position: relative;
      margin: ${theme.space(0.5)} 0;
    `,
    select: css`
      ${theme.body1};
      width: 100%;
      height: 100%;
      appearance: none;
      outline: none;
      border: none;
      background: none;
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border-radius: ${theme.borderRadius};
      cursor: pointer;

      &:disabled {
        cursor: not-allowed;
      }
    `,
    selectFilled: css`
      background-color: ${mixWithSurface(bland.decorative, 0.8)};
      color: ${readableColor(surface)};
      transition: background-color ${theme.duration.standard};

      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.92)};
        color: ${readableColor(surface)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
        color: ${readableColor(surface)};
      }
      &:disabled {
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }
    `,
    selectFilledHasError: css`
      background-color: ${mixWithSurface(danger.decorative, 0.9)};
      color: ${readableColor(surface)};
      &:focus {
        background-color: ${mixWithSurface(danger.decorative, 0.85)};
      }
      &:not([disabled]):hover {
        background-color: ${mixWithSurface(danger.decorative, 0.87)};
      }
    `,
    selectOutlined: css`
      background-color: ${surface};
      transition: border ${theme.duration.standard},
        background-color ${theme.duration.standard},
        color ${theme.duration.standard};
      border: 2px solid ${bland.decorative};
      color: ${readableColor(surface)};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${mixWithSurface(color.decorative, 0.93)};
      }
      &:hover {
        border: 2px solid ${transparentize(color.decorative, 0.3)};
      }
      &:disabled {
        border: 2px solid ${transparentize(bland.decorative, 0.7)};
        background-color: ${mixWithSurface(bland.decorative, 0.9)};
      }
    `,
    selectedOutlinedHasError: css`
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${mixWithSurface(danger.decorative, 0.93)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${transparentize(danger.decorative, 0.3)};
      }
    `,
    icon: css`
      pointer-events: none;
      width: ${theme.space(1)};
      height: ${theme.space(1)};
      position: absolute;
      right: ${theme.space(0.75)};
      top: 50%;
      transform: translateY(-50%);
    `
  };
});
const Select = forwardRef((props, ref) => {
  var _props$component;

  const formControlContext = useContext(FormControlContext);
  const {
    Root,
    id: incomingId,
    styles,
    component,
    selectRef,
    disabled: incomingDisabled,
    hasError: incomingHasError,
    onFocus,
    onBlur,
    variant = 'outlined',
    ...restOfProps
  } = useStyles$e(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
  const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
  const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;

  const handleFocus = e => {
    if (onFocus) {
      onFocus(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(true);
    }
  };

  const handleBlur = e => {
    if (onBlur) {
      onBlur(e);
    }

    if (formControlContext) {
      formControlContext.setFocused(false);
    }
  };

  return /*#__PURE__*/React.createElement(Root, {
    ref: ref
  }, /*#__PURE__*/React.createElement("select", _extends({
    id: id,
    ref: selectRef,
    className: classNames(styles.select, {
      [styles.selectFilled]: variant === 'filled',
      [styles.selectFilledHasError]: variant === 'filled' && hasError,
      [styles.selectOutlined]: variant === 'outlined',
      [styles.selectedOutlinedHasError]: variant === 'outlined' && hasError
    }),
    disabled: disabled,
    onFocus: handleFocus,
    onBlur: handleBlur
  }, restOfProps)), /*#__PURE__*/React.createElement(AngleDownIcon, {
    className: styles.icon
  }));
});
Select.displayName = 'Select';

const useStyles$f = createStyles(({
  css,
  theme,
  color,
  surface
}) => {
  const mixWithSurface = (color, amount) => mix(color, surface, amount);

  return {
    // button base styles
    root: css`
      ${theme.caption}
      font-weight: bold;

      display: inline-flex;
      justify-content: center;
      align-items: center;

      appearance: none;
      outline: none;
      background: transparent;
      border: none;
      text-decoration: none;
      border-radius: 999999px;
      min-height: 34px;

      &:disabled {
        cursor: not-allowed;
      }

      padding: ${theme.space(0.5)} ${theme.space(0.75)};

      & > *:not(:last-child) {
        margin-right: ${theme.space(0.5)};
      }

      & > .hui-button {
        margin-top: ${`-${theme.space(0.5)}`};
        margin-bottom: ${`-${theme.space(0.5)}`};
        margin-right: ${`-${theme.space(0.75)}`};
        margin-left: ${`-${theme.space(0.25)}`};
      }
    `,
    // variants
    outlined: css`
      border: 1px solid ${color.readable};
      color: ${color.readable};

      transition: background-color ${theme.duration.standard},
        border ${theme.duration.standard};

      &:disabled {
        &,
        &:hover,
        &:focus {
          color: ${mixWithSurface(color.readable, 0.4)};
          background-color: transparent;
        }
      }
    `,
    outlinedClickable: css`
      cursor: pointer;
      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.92)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.9)};
      }
      &:active {
        background-color: ${mixWithSurface(color.decorative, 0.8)};
      }
    `,
    filled: css`
      background-color: ${color.decorative};
      color: ${readableColor(color.decorative)};
      border: 2px solid transparent;
      transition: background-color ${theme.duration.standard},
        border ${theme.duration.standard};

      &:disabled {
        background-color: ${mixWithSurface(color.decorative, 0.5)};
        border: 2px solid transparent;
        color: ${mixWithSurface(readableColor(color.decorative), 0.2)};
      }
    `,
    filledClickable: css`
      cursor: pointer;
      &:focus {
        background-color: ${mixWithSurface(color.decorative, 0.18)};
        border: 2px solid ${transparentize('#fff', 0.7)};
      }
      &:hover {
        background-color: ${mixWithSurface(color.decorative, 0.23)};
      }
      &:active {
        background-color: ${mixWithSurface(color.decorative, 0.3)};
        border: 2px solid ${transparentize('#fff', 0.5)};
      }
    `
  };
});
const Chip = forwardRef((props, ref) => {
  var _props$component, _props$color, _props$surface;

  const component = ((_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : props.clickable) ? 'button' : 'div';
  const {
    Root,
    styles,
    component: _component,
    clickable,
    variant = 'outlined',
    ...restOfProps
  } = useStyles$f(props, component);
  const colorContext = useColorContext();
  let color = (_props$color = props.color) !== null && _props$color !== void 0 ? _props$color : colorContext === null || colorContext === void 0 ? void 0 : colorContext.color.original;
  let surface = (_props$surface = props.surface) !== null && _props$surface !== void 0 ? _props$surface : colorContext === null || colorContext === void 0 ? void 0 : colorContext.surface;

  if (variant === 'filled') {
    surface = color;
    color = readableColor(surface);
  }

  return /*#__PURE__*/React.createElement(ColorContextProvider, {
    surface: surface,
    color: color
  }, /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.outlined]: variant === 'outlined',
      [styles.outlinedClickable]: variant === 'outlined' && clickable,
      [styles.filled]: variant === 'filled',
      [styles.filledClickable]: variant === 'filled' && clickable
    }),
    ref: ref
  }, restOfProps)));
});
Chip.displayName = 'Chip';

const useStyles$g = createStyles(({
  css,
  theme
}) => ({
  root: css`
    width: ${theme.space(1.5)};
    height: ${theme.space(1.5)};
    margin-top: ${`-${theme.space(0.5)}`};
    margin-bottom: ${`-${theme.space(0.5)}`};
    margin-left: ${`-${theme.space(0.5)}`};
    border-radius: 99999px;
    overflow: hidden;

    display: flex;
    align-items: center;
    justify-content: center;

    & > img {
      width: 100%;
      height: 100%;
      object-fit: cover;
    }

    & > svg {
      width: ${theme.space(1)};
      height: ${theme.space(1)};
    }
  `
}));
const ChipThumbnail = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    ...restOfProps
  } = useStyles$g(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
ChipThumbnail.displayName = 'ChipThumbnail';

const TimesIcon = forwardRef((props, ref) => {
  return /*#__PURE__*/React.createElement("svg", _extends({
    ref: ref,
    "aria-hidden": "true",
    viewBox: "0 0 352 512"
  }, props), /*#__PURE__*/React.createElement("path", {
    fill: "currentColor",
    d: "M242.72 256l100.07-100.07c12.28-12.28 12.28-32.19 0-44.48l-22.24-22.24c-12.28-12.28-32.19-12.28-44.48 0L176 189.28 75.93 89.21c-12.28-12.28-32.19-12.28-44.48 0L9.21 111.45c-12.28 12.28-12.28 32.19 0 44.48L109.28 256 9.21 356.07c-12.28 12.28-12.28 32.19 0 44.48l22.24 22.24c12.28 12.28 32.2 12.28 44.48 0L176 322.72l100.07 100.07c12.28 12.28 32.2 12.28 44.48 0l22.24-22.24c12.28-12.28 12.28-32.19 0-44.48L242.72 256z"
  }));
});
TimesIcon.displayName = 'TimesIcon';

function delay(milliseconds) {
  return new Promise(resolve => setTimeout(() => resolve('delay'), milliseconds));
}

function useDebounce(value, delay) {
  // State and setters for debounced value
  const [debouncedValue, setDebouncedValue] = useState(value);
  useEffect(() => {
    // Update debounced value after delay
    const handler = setTimeout(() => {
      setDebouncedValue(value);
    }, delay); // Cancel the timeout if value changes (also on delay change or unmount)
    // This is how we prevent debounced value from updating if value is changed ...
    // .. within the delay period. Timeout gets cleared and restarted.

    return () => {
      clearTimeout(handler);
    };
  }, [value, delay] // Only re-call effect if value or delay changes
  );
  return debouncedValue;
}

const useStyles$h = createStyles(({
  css,
  theme,
  surface
}) => ({
  root: css`
    ${theme.caption};
    position: absolute;
    pointer-events: auto;
    padding: ${theme.space(0.5)} ${theme.space(1)};
    min-width: ${theme.block(1)};
    text-align: center;
    color: ${readableColor(readableColor(surface))};
    background-color: ${mix(readableColor(surface), surface, 0.2)};
    z-index: ${theme.zIndex.tooltip};
  `,
  top: css`
    transform: translate(-50%, -100%);
  `,
  left: css`
    transform: translate(-100%, -50%);
  `,
  bottom: css`
    transform: translate(-50%, 0);
  `,
  right: css`
    transform: translate(0, -50%);
  `,
  tooltipContainer: css`
    position: fixed;
    pointer-events: none;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
  `
}));

function Tooltip(props) {
  const {
    Root,
    styles,
    title,
    children,
    position = 'top'
  } = useStyles$h(props);
  const rootRef = useRef(null);
  const [isMouseOverRoot, setIsMouseOverRoot] = useState(false);
  const [isMouseOverTooltip, setIsMouseOverTooltip] = useState(false); // TODO: this would be better as one object

  const [top, setTop] = useState(0);
  const [left, setLeft] = useState(0);
  const [width, setWidth] = useState(0);
  const [height, setHeight] = useState(0);
  const margin = 8;
  let tooltipTop;
  let tooltipLeft;
  const topCenter = top + height / 2;
  const leftCenter = left + width / 2;

  if (position === 'left') {
    tooltipTop = topCenter;
    tooltipLeft = left - margin;
  } else if (position === 'right') {
    tooltipTop = topCenter;
    tooltipLeft = left + width + margin;
  } else if (position === 'bottom') {
    tooltipTop = top + height + margin;
    tooltipLeft = leftCenter;
  } else {
    // position === 'top'
    tooltipTop = top - margin;
    tooltipLeft = leftCenter;
  }

  const [tooltipContainer, setTooltipContainer] = useState(null);
  const isMouseOver = isMouseOverRoot || isMouseOverTooltip;
  const debouncedMouseOver = useDebounce(isMouseOver, 500); // remove the tooltip container on scroll

  useEffect(() => {
    const handleScroll = () => {
      setTooltipContainer(null);
    };

    document.addEventListener('scroll', handleScroll);
    return () => {
      document.removeEventListener('scroll', handleScroll);
    };
  }, []); // updates the coordinates of the tooltip

  useEffect(() => {
    let element = rootRef.current;
    if (!element) return;

    (async () => {
      while (debouncedMouseOver && !!tooltipContainer && !!element) {
        element = rootRef.current;
        if (!element) continue;
        const {
          top,
          left,
          width,
          height
        } = element.getBoundingClientRect();
        setTop(top);
        setLeft(left);
        setWidth(width);
        setHeight(height);
        await delay(100);
      }
    })();
  }, [debouncedMouseOver, rootRef, tooltipContainer]); // adds container to the DOM when the user mouses over

  useEffect(() => {
    if (!debouncedMouseOver) return;
    const tooltipContainer = document.createElement('div');
    tooltipContainer.classList.add(styles.tooltipContainer);
    document.body.appendChild(tooltipContainer);
    setTooltipContainer(tooltipContainer);
    return () => {
      document.body.removeChild(tooltipContainer);
      setTooltipContainer(null);
    };
  }, [debouncedMouseOver, styles]);

  const handleEnterRoot = () => {
    setIsMouseOverRoot(true);
  };

  const handleLeaveRoot = () => {
    setIsMouseOverRoot(false);
  };

  const handleTooltipEnter = () => {
    setIsMouseOverTooltip(true);
  };

  const handleTooltipLeave = () => {
    setIsMouseOverTooltip(false);
  };

  return /*#__PURE__*/React.createElement(React.Fragment, null, children({
    ref: rootRef,
    onMouseEnter: handleEnterRoot,
    onMouseLeave: handleLeaveRoot
  }), tooltipContainer && createPortal( /*#__PURE__*/React.createElement(Root, {
    className: classNames({
      [styles.top]: position === 'top',
      [styles.bottom]: position === 'bottom',
      [styles.left]: position === 'left',
      [styles.right]: position === 'right'
    }),
    onMouseEnter: handleTooltipEnter,
    onMouseLeave: handleTooltipLeave,
    style: {
      top: tooltipTop,
      left: tooltipLeft
    }
  }, title), tooltipContainer));
}

const useStyles$i = createStyles(({
  css,
  theme
}) => ({
  // table container base styles
  root: css`
    border-radius: ${theme.borderRadius};
    margin: ${theme.space(1)} ${theme.space(0.5)};
    box-shadow: 0px 2px 6px 0px ${theme.bland};
  `,
  responsive: css`
    overflow: auto;
  `
}));
const TableContainer = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    responsive = true,
    ...restOfProps
  } = useStyles$i(props, 'div');
  return /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.responsive]: responsive
    }),
    ref: ref
  }, restOfProps));
});
TableContainer.displayName = 'TableContainer';

const useStyles$j = createStyles(({
  css,
  theme,
  surface
}) => ({
  // table base styles
  root: css`
    ${theme.body2}
    color: ${readableColor(surface)};
    border-spacing: ${theme.space(0.5)};
    width: 100%;
    min-width: 500px;
    border-collapse: collapse;
    text-align: left;
    
    th, td {
      background-color: ${surface};
      border-bottom: 1px solid ${theme.bland};
      padding: ${theme.space(1)}
    }
  `,
  // variants
  contained: css`
    th,
    td {
      border: 1px solid ${theme.bland};
    }
  `,
  striped: css`
    tbody tr:nth-child(odd) {
      th,
      td {
        background-color: ${mix(theme.bland, surface, 0.85)};
      }
    }
    th,
    td {
      border: none;
    }
  `,
  ghost: css`
    th,
    td {
      border: none;
    }
  `,
  // Other props
  stickyFirstColumn: css`
    tr > :first-child {
      position: sticky;
      left: 0;
      z-index: 1;
    }
  `
}));
const Table = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    variant = 'outlined',
    stickyFirstColumn = true,
    ...restOfProps
  } = useStyles$j(props, 'table');
  const {
    color,
    surface
  } = useColorContext(props);
  return /*#__PURE__*/React.createElement(ColorContextProvider, {
    color: color.original,
    surface: surface
  }, /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.contained]: variant === 'contained',
      [styles.striped]: variant === 'striped',
      [styles.ghost]: variant === 'ghost',
      [styles.stickyFirstColumn]: stickyFirstColumn
    }),
    ref: ref
  }, restOfProps)));
});
Table.displayName = 'Table';

const useStyles$k = createStyles(({
  css,
  theme
}) => ({
  // table caption base styles
  root: css`
    ${theme.caption}
    display: table-caption;
    text-align: center;
    margin: ${theme.space(1)};
    caption-side: bottom;
  `
}));
const TableCaption = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    ...restOfProps
  } = useStyles$k(props, 'caption');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
TableCaption.displayName = 'TableCaption';

const useStyles$l = createStyles(({
  css
}) => ({
  // table head base styles
  root: css`
    th {
      font-weight: 600;
    }
  `
}));
const TableHead = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    ...restOfProps
  } = useStyles$l(props, 'thead');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
TableHead.displayName = 'TableHead';

const TableBody = 'tbody';

const useStyles$m = createStyles(({
  css,
  color,
  surface
}) => ({
  // table row base styles
  root: css`
    > :first-child {
      min-width: 240px;
    }
  `,
  hoverable: css`
    :hover {
      th,
      td {
        background-color: ${mix(color.decorative, surface, 0.75)} !important;
        transition: background-color 0.2s;
      }
    }
  `
}));
const TableRow = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    hoverable = false,
    ...restOfProps
  } = useStyles$m(props, 'tr');
  return /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.hoverable]: hoverable
    }),
    ref: ref
  }, restOfProps));
});
TableRow.displayName = 'TableRow';

const useStyles$n = createStyles(({
  css
}) => ({
  // table header cell base styles
  root: css`
    font-weight: normal;
    min-width: 120px;
  `
}));
const TableHeaderCell = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    ...restOfProps
  } = useStyles$n(props, 'th');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
TableHeaderCell.displayName = 'TableHeaderCell';

const TableBodyCell = 'td';

const useStyles$o = createStyles(({
  css,
  theme,
  staticVar
}) => ({
  root: css`
    max-height: 90%;
    width: ${theme.block(8)};
    max-width: 100%;
    background-color: ${theme.surface};
    color: ${readableColor(theme.surface)};
    z-index: ${theme.zIndex.modal};
    margin: auto;
    overflow: hidden;
    display: flex;
    flex-direction: column;
    z-index: ${theme.zIndex.modal};
  `,
  container: css`
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    display: flex;
  `,
  backdrop: css`
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    background-color: ${staticVar(transparentize('black', 0.5))};
    z-index: ${theme.zIndex.modal};
  `
}));
const Modal = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    open,
    component: _component,
    // TODO: in the future, we should bind to the escape button for closing
    onClose,
    ...restOfProps
  } = useStyles$o(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'section');
  const [container, setContainer] = useState(null); // TODO: animations would be a nice-to-have

  useEffect(() => {
    if (!open) return;
    const container = document.createElement('div');
    container.classList.add(styles.container);
    document.body.appendChild(container);
    setContainer(container);
    return () => {
      document.body.removeChild(container);
      setContainer(null);
    };
  }, [open, props.style, styles.container]);
  useEffect(() => {
    if (!container) return;

    for (const [k, v] of Object.entries(styles.cssVariableObject)) {
      container.style.setProperty(k, v);
    }
  }, [container, styles.cssVariableObject]); // TODO: throw something in the DOM later for SSR SEO

  return container && createPortal( /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: styles.backdrop,
    onClick: onClose
  }), /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps))), container);
});
Modal.displayName = 'Modal';

const useStyles$p = createStyles(({
  css,
  theme
}) => ({
  root: css`
    display: flex;
    flex-direction: column;
    overflow: hidden;
    flex: 0 0 auto;
    padding: ${theme.space(1)} 0;
  `
}));
const ModalHeader = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    ...restOfProps
  } = useStyles$p(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'header');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
ModalHeader.displayName = 'ModalHeader';

const useStyles$q = createStyles(({
  css
}) => ({
  root: css`
    flex: 1 1 auto;
    position: relative;
    min-height: 50vh;
    overflow: hidden;
    display: flex;
    flex-direction: column;
  `
}));
const ModalContent = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    ...restOfProps
  } = useStyles$q(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
ModalContent.displayName = 'ModalContent';

const useStyles$r = createStyles(({
  css,
  theme
}) => ({
  root: css`
    display: flex;
    flex-direction: column;
    overflow: hidden;
    flex: 0 0 auto;
    padding-bottom: ${theme.space(1)};
  `
}));
const ModalFooter = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    ...restOfProps
  } = useStyles$r(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'footer');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
ModalFooter.displayName = 'ModalFooter';

const useStyles$s = createStyles(({
  css,
  theme
}) => ({
  root: css`
    display: flex;
    justify-content: flex-end;
    padding: 0 ${theme.space(1)};
    margin-top: ${theme.space(1)};
    & > *:not(:first-child) {
      margin-left: ${theme.space(1)};
    }
  `
}));
const ModalActions = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component,
    ...restOfProps
  } = useStyles$s(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps));
});
ModalActions.displayName = 'ModalActions';

const useStyles$t = createStyles(({
  css,
  theme
}) => ({
  root: css`
    height: 1em;
    line-height: 1;
    width: 1.7em;
    display: inline-flex;
    position: relative;
    overflow: visible;
  `,
  wrapper: css`
    font-size: 1.7em;
    position: absolute;
    top: 50%;
    transform: translateY(-50%);
  `
}));
const Emoji = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    children,
    ...restOfProps
  } = useStyles$t(props, 'span');
  return /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps), /*#__PURE__*/React.createElement("span", {
    className: styles.wrapper
  }, children));
});
Emoji.displayName = 'Emoji';

const useStyles$u = createStyles(({
  css,
  theme
}) => ({
  root: css`
    position: absolute;
    width: ${theme.block(2)};
    box-shadow: ${theme.shadows.subtle};
    background-color: ${theme.surface};
    z-index: ${theme.zIndex.modal};
  `,
  top: css`
    transform: translate(-50%, -100%);
  `,
  left: css`
    transform: translate(-100%, -50%);
  `,
  bottom: css`
    transform: translate(-50%, 0);
  `,
  right: css`
    transform: translate(0, -50%);
  `,
  container: css`
    position: fixed;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
    display: flex;
  `,
  backdrop: css`
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    z-index: ${theme.zIndex.modal};
  `
}));
const Menu = forwardRef((props, ref) => {
  const {
    Root,
    styles,
    anchorEl,
    onClose,
    component,
    position = 'bottom',
    children,
    ...restOfProps
  } = useStyles$u(props, props.component ? props.component : 'div');
  const [container, setContainer] = useState(null);
  const [boundingBox, setBoundingBox] = useState({
    top: 0,
    left: 0,
    width: 0,
    height: 0
  });
  let top;
  let left;
  const topCenter = boundingBox.top + boundingBox.height / 2;
  const leftCenter = boundingBox.left + boundingBox.width / 2; // TODO: possibly add other positions:
  // ellipsis menus with icons buttons look bad

  if (position === 'left') {
    top = topCenter;
    left = boundingBox.left;
  } else if (position === 'right') {
    top = topCenter;
    left = boundingBox.left + boundingBox.width;
  } else if (position === 'bottom') {
    top = boundingBox.top + boundingBox.height;
    left = leftCenter;
  } else {
    top = boundingBox.top;
    left = leftCenter;
  } // TODO: animations would be nice


  useEffect(() => {
    if (!anchorEl) return;
    const container = document.createElement('div');
    container.classList.add(styles.container);
    document.body.appendChild(container);
    setContainer(container);
    const {
      top,
      left,
      width,
      height
    } = anchorEl.getBoundingClientRect();
    setBoundingBox({
      top,
      left,
      width,
      height
    });
    return () => {
      document.body.removeChild(container);
      setContainer(null);
    };
  }, [anchorEl, styles.container]);
  return container && createPortal( /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: styles.backdrop,
    onClick: onClose
  }), /*#__PURE__*/React.createElement(Root, _extends({
    className: classNames({
      [styles.top]: position === 'top',
      [styles.bottom]: position === 'bottom',
      [styles.left]: position === 'left',
      [styles.right]: position === 'right'
    }),
    ref: ref,
    style: {
      top,
      left
    }
  }, restOfProps), children)), container);
});
Menu.displayName = 'Menu';

function useMediaQuery(query) {
  const normalizedQuery = query.startsWith('@media') ? query.substring('@media'.length, query.length) : query;
  const mediaQueryList = useMemo(() => matchMedia(normalizedQuery), [normalizedQuery]);
  const [matches, setMatches] = useState(mediaQueryList.matches);
  useEffect(() => {
    const handleChange = e => {
      setMatches(e.matches);
    };

    mediaQueryList.addListener(handleChange);
    return () => {
      mediaQueryList.removeListener(handleChange);
    };
  }, [mediaQueryList]);
  return matches;
}

const useStyles$v = createStyles(({
  css,
  theme
}) => ({
  root: css`
    background-color: ${theme.surface};
    box-shadow: ${theme.shadows.standard};
    width: ${theme.block(3)};
    max-width: 100%;
    height: 100%;
    z-index: ${theme.zIndex.drawer};
  `,
  container: css`
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    display: flex;
  `,
  backdrop: css`
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    background-color: ${transparentize('black', 0.5)};
    z-index: ${theme.zIndex.drawer};
  `
}));
const Drawer = forwardRef((props, ref) => {
  var _props$component;

  const {
    Root,
    styles,
    component: _component,
    open,
    onClose,
    ...restOfProps
  } = useStyles$v(props, (_props$component = props.component) !== null && _props$component !== void 0 ? _props$component : 'div');
  const [container, setContainer] = useState(null); // TODO: animations would be a nice-to-have

  useEffect(() => {
    if (!open) return;
    const container = document.createElement('div');
    container.classList.add(styles.container);
    document.body.appendChild(container);
    setContainer(container);
    return () => {
      document.body.removeChild(container);
      setContainer(null);
    };
  }, [open, styles.container]);
  useEffect(() => {
    if (!container) return;

    for (const [k, v] of Object.entries(styles.cssVariableObject)) {
      container.style.setProperty(k, v);
    }
  }, [container, styles.cssVariableObject]); // TODO: SSR/SEO audit

  return container && createPortal( /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: styles.backdrop,
    onClick: onClose
  }), /*#__PURE__*/React.createElement(Root, _extends({
    ref: ref
  }, restOfProps))), container);
});
Drawer.displayName = 'Drawer';

export { Anchor, Button, CheckIcon, Checkbox, Chip, ChipThumbnail, CircleIcon, Drawer, Emoji, FormControl, HelperText, Label, List, ListItem, ListItemButton, Menu, Modal, ModalActions, ModalContent, ModalFooter, ModalHeader, Radio, RadioGroup, Select, Switch, Table, TableBody, TableBodyCell, TableCaption, TableContainer, TableHead, TableHeaderCell, TableRow, TextArea, TextInput, ThemeProvider, TimesIcon, Tooltip, createTheme, defaultTheme, useMediaQuery };
//# sourceMappingURL=bundle.esm.js.map
